﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks.DistributionGenerator {
  public class DistributionGenerator : NamedItem {

    public DistributionType state = DistributionType.Test;

    private Dictionary<string, IList<double>> data;
    private Dictionary<string, Dictionary<DistributionType, Distribution>> variables;

    private Dictionary<string, StepsDistribution> stepDistributions;
    private Dictionary<string, Distribution> randomDistributions;

    private bool hasNext;
    public bool HasNext {
      get {
        return hasNext;
      }
    }

    public DistributionGenerator(Dictionary<string, IList<double>> data,
    Dictionary<string, Dictionary<DistributionType, Distribution>> variables) {
      this.data = data;
      this.variables = variables;

      spepareteDistributions();
    }

    #region cloning
    protected DistributionGenerator(DistributionGenerator original, Cloner cloner)
      : base(original, cloner) {
      data = original.data;
      state = original.state;
      variables = original.variables;
      stepDistributions = original.stepDistributions;
      randomDistributions = original.randomDistributions;
      hasNext = original.hasNext;
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DistributionGenerator(this, cloner);
    }
    #endregion

    private void spepareteDistributions() {
      foreach (var dic in variables) {
        if (dic.Value[state] is StepsDistribution) {
          stepDistributions[dic.Key] = (StepsDistribution)dic.Value[state];
        } else {
          randomDistributions[dic.Key] = dic.Value[state];
        }
      }
      //new Values can be generated
      hasNext = true;
    }

    public DistributionType ChangeDistributionType(DistributionType type) {
      state = type;
      spepareteDistributions();
      return state;
    }

    public void generateValues() {
      if (hasNext) {
        foreach (var randomDis in randomDistributions) {
          data[randomDis.Key].Add(randomDis.Value.Next());
        }
        if (stepDistributions.Count > 0) {
          bool newValue = false;
          foreach (var stepDis in stepDistributions) {
            if (!newValue && stepDis.Value.HasNext()) {
              data[stepDis.Key].Add(stepDis.Value.Next());
              newValue = true;
              continue;
            }

            if (!stepDis.Value.HasNext())
              stepDis.Value.Reset();

            data[stepDis.Key].Add(stepDis.Value.Current());
          }

          hasNext = !newValue;
        }
      }
    }
  }
}
