﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using HeuristicLab.Common;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public class StepsDistribution : Distribution {

    private double step;
    public double Step {
      get {
        return step;
      }
    }

    private Double curStep;

    public StepsDistribution(DoubleRange range, double step)
      : base(range.Start, range.End) {
      this.step = step;
      curStep = start;
    }

    public StepsDistribution(double start, double end, double step)
      : base(start, end) {
      this.step = step;
    }

    #region cloning
    protected StepsDistribution(StepsDistribution original, Cloner cloner)
      : base(original, cloner) {
      step = original.step;
      curStep = original.curStep;
    }

    public override Common.IDeepCloneable Clone(Common.Cloner cloner) {
      return new StepsDistribution(this, cloner);
    }
    #endregion

    public void Reset() {
      curStep = start;
    }

    public bool HasNext() {
      return curStep <= end;
    }

    public Double Current() {
      return curStep;
    }

    public override Double Next() {
      Double helpStep = curStep;
      curStep += step;
      return helpStep;
    }
  }
}
